#include <vtkVersion.h>

#include <vtkAppendFilter.h>
#include <vtkClipDataSet.h>
#include <vtkDelaunay3D.h>
#include <vtkImplicitPolyDataDistance.h>
#include <vtkSmartPointer.h>
#include <vtkSphereSource.h>
#include <vtkUnstructuredGrid.h>
#include <vtkXMLUnstructuredGridWriter.h>

int main (int, char *[])
{
  // Create a sphere0
  vtkSmartPointer<vtkSphereSource> sphere0 =
    vtkSmartPointer<vtkSphereSource>::New();
  sphere0->SetCenter(0.0, 0.0, 0.0);
  sphere0->SetRadius(0.5);
  sphere0->SetThetaResolution(20);
  sphere0->SetPhiResolution(20);
  sphere0->Update();

  // Create a smaller sphere1
  vtkSmartPointer<vtkSphereSource> sphere1 =
    vtkSmartPointer<vtkSphereSource>::New();
  sphere1->SetCenter(0.0, 0.0, 0.0);
  sphere1->SetRadius(0.4);
  sphere1->SetThetaResolution(20);
  sphere1->SetPhiResolution(20);
  sphere1->Update();

  // Combine the two spheres
  vtkSmartPointer<vtkAppendFilter> appendFilter =
    vtkSmartPointer<vtkAppendFilter>::New();
  appendFilter->AddInputData(sphere0->GetOutput());
  appendFilter->AddInputData(sphere1->GetOutput());
  appendFilter->MergePointsOn();
  appendFilter->Update();

  // Generate a tetrahedral mesh from the input points. By
  // default, the generated volume is the convex hull of the points.
  vtkSmartPointer<vtkDelaunay3D> delaunay3D =
    vtkSmartPointer<vtkDelaunay3D>::New();
  delaunay3D->SetInputConnection(appendFilter->GetOutputPort());
  delaunay3D->Update();

  // Implicit function that will be used to slice the mesh
  vtkSmartPointer<vtkImplicitPolyDataDistance> implicitPolyDataDistance =
    vtkSmartPointer<vtkImplicitPolyDataDistance>::New();
  implicitPolyDataDistance->SetInput(sphere1->GetOutput());

  // Use vtkClipDataSet to slice the grid with the polydata
  vtkSmartPointer<vtkClipDataSet> clipper =
    vtkSmartPointer<vtkClipDataSet>::New();
  clipper->SetInputConnection(delaunay3D->GetOutputPort());
  clipper->SetClipFunction(implicitPolyDataDistance);
  clipper->InsideOutOff();
  clipper->SetValue(0.0);
  clipper->Update();

  // Write file
  vtkSmartPointer<vtkXMLUnstructuredGridWriter> writer =
    vtkSmartPointer<vtkXMLUnstructuredGridWriter>::New();
  writer->SetFileName("volume.vtu");
  writer->SetInputData(clipper->GetOutput());
  writer->Write();

  return EXIT_SUCCESS;
}
